/*!
** @file
** @brief   The implementation of the facade for the clipboard.
** @remarks The facade simplifies the work with the clipboard.
** @author  UETA, Shin-ichi <ueta@usdesign.jp>
** @license The MIT License
** @verbatim

Copyright (C) 2014 Ueta System Design Studio

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

@endverbatim
*/
#pragma once

/*!
** @brief   The facade for the clipboard.
** @remarks -
*/
class CClipboard
{
public:

	/*!
	** @brief   Allocates a new global memory object and copies the specified string into it.
	** @param   [in] psz the string to copy.
	** @returns a handle to the newly allocated memory object if successful, or NULL otherwise.
	** @remarks -
	*/
	static HGLOBAL GlobalAllocString(const wchar_t* psz)
	{
		size_t cch = wcslen(psz) + 1; // including the terminating null character

		if (HGLOBAL hGlobal = ::GlobalAlloc(GMEM_MOVEABLE, sizeof(wchar_t) * cch))
		{
			if (wchar_t* pszGlobal = static_cast<wchar_t*>(::GlobalLock(hGlobal)))
			{
				ATLVERIFY(wcscpy_s(pszGlobal, cch, psz) == 0);

				ATLVERIFY(::GlobalUnlock(hGlobal) == 0);
				ATLASSERT(::GetLastError() == NO_ERROR);

				return hGlobal;
			}
			ATLVERIFY(::GlobalFree(hGlobal) == NULL);
		}
		return NULL;
	}

	/*!
	** @brief   Deallocates a string allocated in the global memory.
	** @param   [in] hGlobal a handle to the global memory object.
	** @returns TRUE if successful, or FALSE otherwise.
	** @remarks -
	*/
	static BOOL GlobalFreeString(HGLOBAL hGlobal)
	{
		if (::GlobalFree(hGlobal) == NULL)
		{
			return TRUE;
		}
		return FALSE;
	}

	/*!
	** @brief   Clears the clipboard and then places the specified data.
	** @param   [in] uFormat the clipboard format.
	** @param   [in] hMem a handle to the data in the specified format.
	** @param   [in] hWndOwner a handle to the window to be associated with the clipboard.
	** @returns TRUE if successful, or FALSE otherwise.
	** @remarks -
	*/
	static BOOL SetData(UINT uFormat, HANDLE hMem, HWND hWndOwner = ::GetDesktopWindow())
	{
		BOOL bSuccess = FALSE;

		if (::OpenClipboard(hWndOwner))
		{
			if (::EmptyClipboard())
			{
				if (::SetClipboardData(uFormat, hMem) != NULL)
				{
					bSuccess = TRUE;
				}
			}
			ATLVERIFY(::CloseClipboard());
		}
		return bSuccess;
	}

	/*!
	** @brief   Clears the clipboard and then places the specified text data.
	** @param   [in] pszText a null-terminated string to set to the clipboard.
	** @param   [in] hWndOwner a handle to the window to be associated with the clipboard.
	** @returns TRUE if successful, or FALSE otherwise.
	** @remarks -
	*/
	static BOOL SetText(const wchar_t* pszText, HWND hWndOwner = ::GetDesktopWindow())
	{
		if (HGLOBAL hGlobal = GlobalAllocString(pszText))
		{
			if (SetData(CF_UNICODETEXT, hGlobal, hWndOwner)) // the system owns the data
			{
				return TRUE;
			}
			ATLVERIFY(GlobalFreeString(hGlobal));
		}
		return FALSE;
	}
};
