/*!
** @file
** @brief   The implementation of the DWM-aware functions.
** @remarks More features for the Desktop Window Manager APIs are provided.
** @author  UETA, Shin-ichi <ueta@usdesign.jp>
** @license The MIT License
** @verbatim

Copyright (C) 2014 Ueta System Design Studio

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

@endverbatim
*/
#pragma once

#pragma comment(lib, "dwmapi.lib")

#include <dwmapi.h>

/*!
** @brief   Retrieves the extended frame bounds of the specified window.
** @param   [in] hWnd a handle to the window.
** @param   [out] lpRect a pointer to a RECT structure to receive the bounds.
** @returns TRUE if successful, or FALSE otherwise.
** @remarks -
*/
BOOL DwmGetExtendedFrameBounds(HWND hWnd, LPRECT lpRect)
{
	HRESULT hResult = ::DwmGetWindowAttribute(hWnd, DWMWA_EXTENDED_FRAME_BOUNDS, lpRect, sizeof(*lpRect));

	return SUCCEEDED(hResult) ? TRUE : FALSE;
}

/*!
** @brief   Retrieves information about the specified window.
** @param   [in] hWnd a handle to the window whose information is to be retrieved.
** @param   [out] pwi a pointer to a WINDOWINFO structure to receive the information.
** @returns TRUE if successful, or FALSE otherwise.
** @remarks -
*/
BOOL DwmGetWindowInfo(HWND hWnd, PWINDOWINFO pwi)
{
	if (::GetWindowInfo(hWnd, pwi))
	{
		#if 1 // to account the extended frame bounds
		{
			CRect& rcWindow = static_cast<CRect&>(pwi->rcWindow);

			CRect rcExtendedFrameBounds;

			if (DwmGetExtendedFrameBounds(hWnd, rcExtendedFrameBounds))
			{
				ATLASSERT(rcExtendedFrameBounds.Width()  >= rcWindow.Width());
				ATLASSERT(rcExtendedFrameBounds.Height() >= rcWindow.Height());

				rcWindow = rcExtendedFrameBounds;
			}
		}
		#endif

		return TRUE;
	}
	return FALSE;
}

/*!
** @brief   Changes the size, position, and Z-order of a window.
** @param   [in] hWnd a handle to the window.
** @param   [in] hWndInsertAfter a handle to the window to precede in the Z-order.
** @param   [in] x the new position of the left side of the window.
** @param   [in] y the new position of the top of the window.
** @param   [in] cx the new width of the window.
** @param   [in] cy the new height of the window.
** @param   [in] uFlags the sizing and positioning flags.
** @returns TRUE if successful, or FALSE otherwise.
** @remarks -
*/
BOOL DwmSetWindowPos(HWND hWnd, HWND hWndInsertAfter, int x, int y, int cx, int cy, UINT uFlags)
{
	#if 1 // to account the extended frame bounds
	{
		CRect rcWindow;

		if (::GetWindowRect(hWnd, rcWindow))
		{
			CRect rcExtendedFrameBounds;

			if (DwmGetExtendedFrameBounds(hWnd, rcExtendedFrameBounds))
			{
				ATLASSERT(rcWindow.left >= rcExtendedFrameBounds.left);
				ATLASSERT(rcWindow.top  >= rcExtendedFrameBounds.top);

				x += rcWindow.left - rcExtendedFrameBounds.left;
				y += rcWindow.top  - rcExtendedFrameBounds.top;

				ATLASSERT(rcExtendedFrameBounds.Width()  >= rcWindow.Width());
				ATLASSERT(rcExtendedFrameBounds.Height() >= rcWindow.Height());

				int dx = rcExtendedFrameBounds.Width()  - rcWindow.Width();
				int dy = rcExtendedFrameBounds.Height() - rcWindow.Height();

				cx -= min(cx, dx);
				cy -= min(cy, dy);
			}
		}
	}
	#endif

	if (::SetWindowPos(hWnd, hWndInsertAfter, x, y, cx, cy, uFlags))
	{
		;

		return TRUE;
	}
	return FALSE;
}
