/*!
** @file
** @brief   The implementation of the About Box.
** @remarks The About Box displays information about the program.
** @author  UETA, Shin-ichi <ueta@usdesign.jp>
** @license The MIT License
** @verbatim

Copyright (C) 2014 Ueta System Design Studio

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

@endverbatim
*/
#pragma once

#pragma comment(lib, "version.lib")

#include <winver.h>

/*!
** @brief   The base class of the site for the About Box.
** @remarks -
*/
class ATL_NO_VTABLE CAboutBoxSite
{
public:

};

/*!
** @brief   The implementation of the About Box.
** @remarks -
*/
class CAboutBox
{
public:

	/*!
	** @brief   The version-information resource returned by the GetFileVersionInfo function.
	** @remarks -
	*/
	class CFileVersionInfo
	{
	private:

		CAutoVectorPtr<BYTE> m_lpData;

		VS_FIXEDFILEINFO* m_lpRootBlock;

		struct TRANSLATION
		{
			WORD wLanguage;
			WORD wCodePage;
		};

		TRANSLATION* m_lpTranslation;

	public:

		/*!
		** @brief   Initializes a new CFileVersionInfo object.
		** @remarks -
		*/
		CFileVersionInfo()
		{
			m_lpRootBlock   = NULL;
			m_lpTranslation = NULL;
		}

		~CFileVersionInfo()
		{
			Free();
		}

		/*!
		** @brief   Retrieves version information for the specified file.
		** @param   [in] lpszFileName the name of the file from which the version-resource is extracted.
		** @returns A standard HRESULT value.
		** @remarks -
		*/
		HRESULT LoadFileVersionInfo(LPCTSTR lpszFileName)
		{
			Free();

			DWORD dwHandle;
			DWORD dwLen = ::GetFileVersionInfoSize(lpszFileName, &dwHandle);

			if (dwLen == 0)
				return AtlHresultFromLastError();
			if (!m_lpData.Allocate(dwLen))
				return AtlHresultFromWin32(ERROR_OUTOFMEMORY);
			if (!::GetFileVersionInfo(lpszFileName, dwHandle, dwLen, m_lpData))
				return AtlHresultFromLastError();

			VS_FIXEDFILEINFO* lpRootBlock = NULL;
			UINT uRootBlockLength;
			ATLVERIFY(::VerQueryValue(m_lpData, TEXT("\\"),
					reinterpret_cast<LPVOID*>(&lpRootBlock), &uRootBlockLength));
			ATLASSERT(lpRootBlock);
			ATLASSERT(uRootBlockLength == UINT(sizeof(VS_FIXEDFILEINFO)));

			m_lpRootBlock = lpRootBlock;

			TRANSLATION* lpTranslation = NULL;
			UINT uTranslationLength;
			ATLVERIFY(::VerQueryValue(m_lpData, TEXT("\\VarFileInfo\\Translation"),
					reinterpret_cast<LPVOID*>(&lpTranslation), &uTranslationLength));
			ATLASSERT(lpTranslation);
			ATLASSERT(uTranslationLength == UINT(sizeof(TRANSLATION) * 2));

			WORD wLanguage = ::GetUserDefaultLangID();

			if (lpTranslation[1].wLanguage == wLanguage)
			{
				m_lpTranslation = &lpTranslation[1];
			}
			else
			{
				wLanguage = MAKELANGID(LANG_ENGLISH, SUBLANG_ENGLISH_US);
				ATLASSERT(lpTranslation[0].wLanguage == wLanguage);
				m_lpTranslation = &lpTranslation[0];
			}
			return S_OK;
		}

		/*!
		** @brief   Retrieves the specified string value from the version-information resource.
		** @param   [in] lpszStringName the name of the predefined string value to be retrieved.
		** @returns A pointer to the string value if it is available, or NULL otherwise.
		** @remarks -
		*/
		LPCTSTR QueryValue(LPCTSTR lpszStringName) const
		{
			ATLENSURE_RETURN_VAL(m_lpData, NULL);

			ATLASSERT(m_lpTranslation);

			CString strSubBlock;
			strSubBlock.Format(TEXT("\\StringFileInfo\\%04x%04x\\%s"),
					m_lpTranslation->wLanguage,
					m_lpTranslation->wCodePage, lpszStringName);

			LPTSTR lpszStringValue = NULL;
			UINT uStringValueLength;
			ATLVERIFY(::VerQueryValue(m_lpData, strSubBlock,
					reinterpret_cast<LPVOID*>(&lpszStringValue), &uStringValueLength));
			ATLASSERT(lpszStringValue);
			ATLASSERT(uStringValueLength == UINT(lstrlen(lpszStringValue) + 1));

			return lpszStringValue;
		}

		/*!
		** @brief   Deallocates the retrieved version-information resource.
		** @returns No return value.
		** @remarks -
		*/
		void Free()
		{
			m_lpRootBlock   = NULL;
			m_lpTranslation = NULL;

			m_lpData.Free();
		}
	};

	/*!
	** @brief   Displays a modal dialog box that contains version information for the specified file.
	** @param   [in] hwndOwner a handle to the owner window of the dialog box to be created.
	** @param   [in] lpszFilename the name of the file from which the version-resource is extracted.
	** @param   [in] caption a null-terminated string, or the ID of a string resource, that contains the title.
	** @param   [in] icon the ID of an icon resource to be displayed.
	** @returns IDOK if successful, or zero otherwise.
	** @remarks -
	*/
	static int AboutBox(HWND hwndOwner, LPCTSTR lpszFilename, _U_STRINGorID caption, _U_STRINGorID icon)
	{
		CFileVersionInfo version;

		HRESULT hResult = version.LoadFileVersionInfo(lpszFilename);

		ATLENSURE_RETURN_VAL(SUCCEEDED(hResult), 0);

		CString strText;
		strText.Format(TEXT("%s %s\n%s"),
				version.QueryValue(TEXT("ProductName")),
				version.QueryValue(TEXT("ProductVersion")),
				version.QueryValue(TEXT("LegalCopyright")));

		MSGBOXPARAMS mbp = {sizeof(MSGBOXPARAMS)};

		mbp.hwndOwner   = hwndOwner;
		mbp.hInstance   = _Module.GetResourceInstance();
		mbp.lpszText    = strText;
		mbp.lpszCaption = caption.m_lpstr;
		mbp.lpszIcon    = icon.m_lpstr;

		if (mbp.lpszIcon)
		{
			mbp.dwStyle |= MB_USERICON;
		}
		ATLVERIFY(::MessageBeep(MB_ICONINFORMATION));

		return ::MessageBoxIndirect(&mbp);
	}

public:

	/*!
	** @brief   Displays a modal dialog box that contains version information.
	** @param   [in] hWndParent a handle to the owner window.
	** @returns IDOK if successful, or zero otherwise. -1 is also reserved as an error code.
	** @remarks -
	*/
	static int DoModal(HWND hWndParent = ::GetActiveWindow())
	{
		int nResult = AboutBox(hWndParent, __targv[0], IDR_APP, IDR_APP);

		return nResult;
	}
};

/*!
** @brief   The implementation to host the About Box.
** @remarks -
*/
template <class T>
class CAboutBoxSiteImpl :
		public CAboutBoxSite,
		public CMessageMap
{
public:

	/*!
	** @brief   Initializes a new CAboutBoxSiteImpl object.
	** @remarks -
	*/
	CAboutBoxSiteImpl()
	{
		;
	}

	/*!
	** @brief   Binds the site for the About Box to the owner.
	** @returns TRUE if successful, or FALSE otherwise.
	** @remarks -
	*/
	BOOL BindAboutBox()
	{
		ATLASSERT((IDM_ABOUT & 0xFFF0) == IDM_ABOUT);

		T* pT = static_cast<T*>(this);

		CString strItem;
		ATLVERIFY(strItem.LoadString(IDM_ABOUT));
		CMenuHandle menu = pT->GetSystemMenu(FALSE);
		ATLASSERT(menu);
		ATLVERIFY(menu.InsertMenu(0, MF_BYPOSITION | MF_STRING, IDM_ABOUT, strItem));
		ATLVERIFY(menu.InsertMenu(1, MF_BYPOSITION | MF_SEPARATOR));

		return TRUE;
	}

	BEGIN_MSG_MAP(CAboutBoxSiteImpl)
		MESSAGE_HANDLER(WM_SYSCOMMAND, OnSysCommand)
	END_MSG_MAP()

	/*!
	** @brief   Handles the WM_SYSCOMMAND message.
	** @param   [in] uMsg specifies the message.
	** @param   [in] wParam additional message-specific information.
	** @param   [in] lParam additional message-specific information.
	** @param   [in, out] bHandled set to FALSE if further processing is needed.
	** @returns The result of the message processing.
	** @remarks -
	*/
	LRESULT OnSysCommand(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		UNREFERENCED_PARAMETER(uMsg);

		switch (UINT(wParam) & 0xFFF0)
		{
		case IDM_ABOUT:
			OnAbout(UINT(wParam), lParam);
			return LRESULT(0);
		default:
			bHandled = FALSE;
			return LRESULT(0);
		}
	}

	/*!
	** @brief   Handles the system command to display the About Box.
	** @param   [in] nID the type of system command requested.
	** @param   [in] lParam additional message-specific information.
	** @returns No return value.
	** @remarks -
	*/
	void OnAbout(UINT nID, LPARAM lParam)
	{
		UNREFERENCED_PARAMETER(nID);
		UNREFERENCED_PARAMETER(lParam);

		ATLVERIFY(CAboutBox::DoModal() == IDOK);
	}
};
