/*!
** @file
** @brief   The customization of the Shell File Open dialog.
** @remarks Either a file or a directory can be selected.
** @author  UETA, Shin-ichi <ueta@usdesign.jp>
** @license The MIT License
** @verbatim

Copyright (C) 2014 Ueta System Design Studio

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

@endverbatim
*/
#pragma once

#pragma warning(push)
#pragma warning(disable: 4100)

///////////////////////////////////////////////////////////////////////// @cond
// CShellFileDialogCustomizeImpl - customizes Shell File dialog

template <class T, class TBase>
class ATL_NO_VTABLE CShellFileDialogCustomizeImpl : public TBase, public IFileDialogControlEvents
{
public:
	CComPtr<IFileDialogCustomize> m_spFileDlgCustomize;

	CShellFileDialogCustomizeImpl()
	{
		if (m_spFileDlg)
			m_spFileDlg->QueryInterface(IID_PPV_ARGS(&m_spFileDlgCustomize));
	}

	// Implementation - IUnknown interface

	STDMETHOD(QueryInterface)(REFIID riid, void** ppvObject)
	{
		T* pT = static_cast<T*>(this);

		if (::IsEqualGUID(riid, IID_IFileDialogControlEvents))
		{
			if (ppvObject == NULL)
				return E_POINTER;
			*ppvObject = static_cast<IFileDialogControlEvents*>(pT);
			// AddRef(); not needed
			return S_OK;
		}
		return TBase::QueryInterface(riid, ppvObject);
	}
	virtual ULONG STDMETHODCALLTYPE AddRef()
	{
		return TBase::AddRef();
	}
	virtual ULONG STDMETHODCALLTYPE Release()
	{
		return TBase::Release();
	}

	// Implementation - IFileDialogControlEvents interface

	virtual HRESULT STDMETHODCALLTYPE OnItemSelected(IFileDialogCustomize* pfdc, DWORD dwIDCtl, DWORD dwIDItem)
	{
		T* pT = static_cast<T*>(this);
		ATLASSERT(m_spFileDlgCustomize.IsEqualObject(pfdc));

		return pT->OnItemSelected(dwIDCtl, dwIDItem);
	}
	virtual HRESULT STDMETHODCALLTYPE OnButtonClicked(IFileDialogCustomize* pfdc, DWORD dwIDCtl)
	{
		T* pT = static_cast<T*>(this);
		ATLASSERT(m_spFileDlgCustomize.IsEqualObject(pfdc));

		return pT->OnButtonClicked(dwIDCtl);
	}
	virtual HRESULT STDMETHODCALLTYPE OnCheckButtonToggled(IFileDialogCustomize* pfdc, DWORD dwIDCtl, BOOL bChecked)
	{
		T* pT = static_cast<T*>(this);
		ATLASSERT(m_spFileDlgCustomize.IsEqualObject(pfdc));

		return pT->OnCheckButtonToggled(dwIDCtl, bChecked);
	}
	virtual HRESULT STDMETHODCALLTYPE OnControlActivating(IFileDialogCustomize* pfdc, DWORD dwIDCtl)
	{
		T* pT = static_cast<T*>(this);
		ATLASSERT(m_spFileDlgCustomize.IsEqualObject(pfdc));

		return pT->OnControlActivating(dwIDCtl);
	}

	// Overrideables - Event handlers

	HRESULT OnItemSelected(DWORD dwIDCtl, DWORD dwIDItem)
	{
		return E_NOTIMPL;
	}
	HRESULT OnButtonClicked(DWORD dwIDCtl)
	{
		return E_NOTIMPL;
	}
	HRESULT OnCheckButtonToggled(DWORD dwIDCtl, BOOL bChecked)
	{
		return E_NOTIMPL;
	}
	HRESULT OnControlActivating(DWORD dwIDCtl)
	{
		return E_NOTIMPL;
	}
};

///////////////////////////////////////////////////////////////////////////////
// CShellFileOpenDialogCustomizeImpl - customizes Shell File Open dialog

template <class T, class TBase = CShellFileOpenDialogImpl<T>>
class ATL_NO_VTABLE CShellFileOpenDialogCustomizeImpl : public CShellFileDialogCustomizeImpl<T, TBase>
{
public:
	CShellFileOpenDialogCustomizeImpl()
	{
		;
	}
};

///////////////////////////////////////////////////////////////////////////////
// CShellFileSaveDialogCustomizeImpl - customizes Shell File Save dialog

template <class T, class TBase = CShellFileSaveDialogImpl<T>>
class ATL_NO_VTABLE CShellFileSaveDialogCustomizeImpl : public CShellFileDialogCustomizeImpl<T, TBase>
{
public:
	CShellFileSaveDialogCustomizeImpl()
	{
		;
	}
};

////////////////////////////////////////////////////////////////////// @endcond

#pragma warning(pop)

/*!
** @brief   The customization of the Shell File Open dialog.
** @remarks -
*/
class CShellFileOpenDialogEx : public CShellFileOpenDialogCustomizeImpl<CShellFileOpenDialogEx>
{
private:

	CComPtr<IShellItem> m_spShellItem;

public:

	/*!
	** @brief   Constructs a new CShellFileOpenDialogEx object with an initial directory.
	** @param   [in] pszInitialDir the initial directory, or NULL for default.
	** @remarks -
	*/
	CShellFileOpenDialogEx(LPCWSTR pszInitialDir = NULL)
	{
		HRESULT hResult = S_OK;

		ATLASSERT(m_spFileDlg);
		ATLASSERT(m_spFileDlgCustomize);

		if (m_spFileDlg)
		{
			CString strTitle;
			ATLVERIFY(strTitle.LoadString(IDC_BROWSE));
			hResult = m_spFileDlg->SetTitle(strTitle);
			ATLASSERT(SUCCEEDED(hResult));

			FILEOPENDIALOGOPTIONS fos = 0;
			hResult = m_spFileDlg->GetOptions(&fos);
			ATLASSERT(SUCCEEDED(hResult));
			hResult = m_spFileDlg->SetOptions(fos | FOS_DONTADDTORECENT | FOS_NOCHANGEDIR | FOS_NODEREFERENCELINKS);
			ATLASSERT(SUCCEEDED(hResult));

			if (pszInitialDir)
			{
				CComPtr<IShellItem> spShellItem;

				hResult = ::SHCreateItemFromParsingName(pszInitialDir, NULL, IID_PPV_ARGS(&spShellItem));

				if (SUCCEEDED(hResult))
				{
					hResult = m_spFileDlg->SetFolder(spShellItem);
					ATLASSERT(SUCCEEDED(hResult));
				}
			}
		}
		if (m_spFileDlgCustomize)
		{
			CString strMargin(TEXT(' '), 5);
			CString strLable;
			ATLVERIFY(strLable.LoadString(IDC_BROWSE_SELECTFOLDER));

			strLable = strMargin + strLable + strMargin;

			hResult = m_spFileDlgCustomize->AddPushButton(IDC_BROWSE_SELECTFOLDER, strLable);
			ATLASSERT(SUCCEEDED(hResult));
			hResult = m_spFileDlgCustomize->SetControlState(IDC_BROWSE_SELECTFOLDER, CDCS_VISIBLE);
			ATLASSERT(SUCCEEDED(hResult));
		}
	}

	/*!
	** @brief   Retrieves the path and file name of the selected file or directory.
	** @param   [out] strFilePath a string that receives the path and file name.
	** @returns A standard HRESULT value.
	** @remarks -
	*/
	HRESULT GetFilePath(CString& strFilePath)
	{
		if (m_spShellItem)
		{
			return GetFileNameFromShellItem(m_spShellItem, SIGDN_FILESYSPATH, strFilePath);
		}
		return CShellFileOpenDialogImpl<CShellFileOpenDialogEx>::GetFilePath(strFilePath);
	}

	/*!
	** @brief   Called when the user specifies a file name and clicks the OK button.
	** @returns A standard HRESULT value.
	** @remarks -
	*/
	HRESULT OnFileOk()
	{
		ATLENSURE_RETURN_HR(m_spFileDlgCustomize, E_NOTIMPL);

		HRESULT hResult = S_OK;

		m_spShellItem.Release();

		return hResult;
	}

	/*!
	** @brief   Called when a new folder is opened.
	** @returns A standard HRESULT value.
	** @remarks -
	*/
	HRESULT OnFolderChange()
	{
		ATLENSURE_RETURN_HR(m_spFileDlgCustomize, E_NOTIMPL);

		HRESULT hResult = S_OK;

		m_spShellItem.Release();

		ATLASSERT(m_spFileDlg);
		hResult = m_spFileDlg->SetFileName(NULL);
		ATLASSERT(SUCCEEDED(hResult));
		ATLASSERT(m_spFileDlgCustomize);
		hResult = m_spFileDlgCustomize->SetControlState(IDC_BROWSE_SELECTFOLDER, CDCS_VISIBLE);
		ATLASSERT(SUCCEEDED(hResult));

		return hResult;
	}

	/*!
	** @brief   Called when the selection changes in the currently opened folder.
	** @returns A standard HRESULT value.
	** @remarks -
	*/
	HRESULT OnSelectionChange()
	{
		ATLENSURE_RETURN_HR(m_spFileDlgCustomize, E_NOTIMPL);

		HRESULT hResult = S_OK;

		CDCONTROLSTATEF cdcs = CDCS_VISIBLE;

		m_spShellItem.Release();

		ATLASSERT(m_spFileDlg);
		hResult = m_spFileDlg->GetCurrentSelection(&m_spShellItem);

		if (SUCCEEDED(hResult))
		{
			SFGAOF sfgaoAttribs = 0;
			ATLASSERT(m_spShellItem);
			hResult = m_spShellItem->GetAttributes(SFGAO_FILESYSTEM | SFGAO_FOLDER | SFGAO_LINK, &sfgaoAttribs);
			ATLASSERT(SUCCEEDED(hResult));

			CString strFilePath;
			LPCTSTR pszFileName = NULL;

			if (sfgaoAttribs & SFGAO_FILESYSTEM)
			{
				hResult = GetFileNameFromShellItem(m_spShellItem, SIGDN_FILESYSPATH, strFilePath);
				ATLASSERT(SUCCEEDED(hResult));
				pszFileName = ATLPath::FindFileName(strFilePath);
				ATLASSERT(pszFileName);
			}
			switch (sfgaoAttribs)
			{
			case SFGAO_FILESYSTEM | SFGAO_FOLDER:
			case SFGAO_FILESYSTEM | SFGAO_FOLDER | SFGAO_LINK:

				cdcs = CDCS_ENABLEDVISIBLE;

				ATLASSERT(m_spFileDlg);
				hResult = m_spFileDlg->SetFileName(pszFileName);
				ATLASSERT(SUCCEEDED(hResult));
				break;

			case SFGAO_FILESYSTEM | SFGAO_LINK:

				ATLASSERT(m_spFileDlg);
				hResult = m_spFileDlg->SetFileName(pszFileName);
				ATLASSERT(SUCCEEDED(hResult));
				break;
			}
		}
		ATLASSERT(m_spFileDlgCustomize);
		hResult = m_spFileDlgCustomize->SetControlState(IDC_BROWSE_SELECTFOLDER, cdcs);
		ATLASSERT(SUCCEEDED(hResult));

		return hResult;
	}

	/*!
	** @brief   Called when the user clicks a command button.
	** @param   [in] dwIDCtl the ID of the button that the user clicked.
	** @returns A standard HRESULT value.
	** @remarks -
	*/
	HRESULT OnButtonClicked(DWORD dwIDCtl)
	{
		ATLENSURE_RETURN_HR(m_spFileDlgCustomize, E_NOTIMPL);

		HRESULT hResult = S_OK;

		switch (dwIDCtl)
		{
		case IDC_BROWSE_SELECTFOLDER:

			ATLASSERT(m_spFileDlg);
			hResult = m_spFileDlg->Close(S_OK);
			ATLASSERT(SUCCEEDED(hResult));
			break;
		}
		return hResult;
	}
};
